import { NetInfo, AsyncStorage, Platform } from 'react-native';

import Download from './download';


export default {
    Upgrade_App,
    Upgrade_Bundle
}


/**
 * info.pid, info.platform, info.versionCode
 * @param {*} info 
 * @param {*} progressFunc 
 */
function Upgrade_App(info, progressFunc) {
    return new Promise((resolve, reject) => {
        var data = {};
        app_ares(info.pid, info.platform, info.versionCode).then(result => {
            if (null == result) {
                resolve(false);
                return;
            }
            data['app_version'] = result['versionCode'];
            data['app_url'] = result['toFile'];

            //当前app版本大于等于接口获取版本，则不需要更新处理
            if (info.versionCode >= data['app_version']) {
                resolve(false);
                return new Promise((resolve, reject) => { resolve(false) });
            } else {
                return Download.download(result['url'], result['toFile'], progressFunc);
            }
        }).then(result => {
            // console.log('vvv =>', info.versionCode, data);
            // let mergeData = Object.assign(result, data);
            AsyncStorage.setItem('rn_render_storage_key', JSON.stringify(data), err => {
                if (err) {
                    resolve(false);
                } else {
                    resolve(true);
                }
            });
        }).catch(e => {
            if (__DEV__) console.error(e);
            reject(e);
        });
    });
}

/**
 * info.pid, info.eid, info.platform, info.sv ,info.version
 * @param {*} info 
 * @param {*} progressFunc 
 */
function Upgrade_Bundle(info, progressFunc) {
    console.log('info ===>', info);
    return new Promise((resolve, reject) => {
        var data = {};
        // let bundleVersionKey = `rn_bundle_${info.pid}_${info.platform}_${info.sv}`;
        let bundleVersionKey = `rn_bundle_${info.pid}_${info.eid}_${info.platform}_${info.sv}`;
        var bunderUnique = null;
        bundle_ares(info.pid, info.eid, info.platform, info.sv).then(result => {
            //获取版本信息
            // url: url,
            // toFile: name,
            // version: version
            console.log('bundle_ares =>', result);

            //版本相同不做更新
            if (null == result 
                || info.version == result['version']) {
                resolve(false);
                return;
            }

            data['bundle_version'] = result['version'];
            data['bundle_url'] = result['toFile'];
            data['url'] = result['url'];
            data['id'] = data['bundle_version'] + ':' + data['bundle_url'];
            bunderUnique = data['id'];
            return AsyncStorage.getItem(bundleVersionKey, (error, result) => {
                console.log(`${bundleVersionKey} :error :${error}  result: ${result}`);
            });
        }).then(aget => {
            console.log(`check =>`, aget, data['id']);
            //对比版本信息，判断是否需要下载
            if (aget != data['id']) {
                //bundle version + url 如果不等 需要重新加载接口中的版本
                return Download.download(data['url'], data['bundle_url'], progressFunc);
                // Download.download(data['url'], data['bundle_url'], progressFunc).then(r => {
                //     return new Promise((resolve, reject) => { resolve(true) });
                // });
            } else {
                return new Promise((resolve, reject) => { resolve(false) });
            }
        }).then(result => {

            if (result) {
                console.log('last =>', result);
                //执行render需要的参数
                // let mergeData = Object.assign(result, data);
                AsyncStorage.setItem('rn_render_storage_key', JSON.stringify(data), err => {
                    if (err) {
                        resolve(false);
                    } else {
                        AsyncStorage.setItem(bundleVersionKey, bunderUnique);
                        resolve(true);
                    }
                });
            } else {
                resolve(false);
            }

        }).catch(e => {
            if (__DEV__) console.error(e);
            reject(e);
        });
    });
}


/**
 * 
 * @param {*} pid 
 * @param {*} platform 
 * @param {*} versionCode 
 */
function app_ares(pid, platform, versionCode) {
    //'http://aresapi.qianmi.com/api/app/version/check?pid=8&platform=a&versionCode=3'
    return app(`http://aresapi.qianmi.com/api/app/version/check?pid=${pid}&platform=${platform}&versionCode=${versionCode}`);
}


/**
 * app升级接口
 */
function app(url) {
    return new Promise((resolve, reject) => {
        Download.getJSON(url).then(response => {
            if (1 != response.status) {
                reject(response.message);
            } else {
                let data = response.data;
                let needforceUpdate = data['needforceUpdate'];
                let lastestVersion = data['lastestVersion'];
                if (null == lastestVersion) {
                    resolve(null);
                    return;
                }
                let projectId = lastestVersion['project']['id'];
                let url = lastestVersion['apkUrl'];
                let addDate = lastestVersion['addDate'];
                let versionCode = lastestVersion['versionCode'];
                var name = Download.tofileName(url);
                resolve({
                    needforceUpdate: needforceUpdate,
                    url: url,
                    versionCode: versionCode,
                    toFile: name,
                    id: projectId,
                    addDate: addDate
                });
            }
        }).catch((error) => {
            if (__DEV__) console.error(error);
            reject(error);
        });
    });
}

/**
 * 
 * @param {*} pid 
 * @param {*} eid 
 * @param {*} platform 
 * @param {*} sv 
 */
function bundle_ares(pid, eid, platform, sv) {
    //http://aresapi.qianmi.com/api/map?pid=8&eid=11&platform=android&sv=1
    return bundle(`http://aresapi.qianmi.com/api/map?pid=${pid}&eid=${eid}&platform=${platform}&sv=${sv}`);
}

/**
 * bundle新版本查询接口
 */
function bundle(url) {
    return new Promise((resolve, reject) => {
        Download.getJSON(url).then(response => {
            if (1 != response.status) {
                reject(response.message);
            } else {
                if (0 === response.data['routes'].length) {
                    resolve(null);
                } else {
                    let route = response.data['routes'][0];
                    let url = route['remote_file'];
                    let version = route['version'];
                    var name = Download.tofileName(url);
                    resolve({
                        url: url,
                        toFile: name,
                        version: version
                    });
                }
            }
        }).catch((error) => {
            if (__DEV__) console.error(error);
            reject(error);
        });
    });
}